import { BigintIsh, CurrencyAmount, Price, Token } from '@uniswap/sdk-core';
import JSBI from 'jsbi';
import { FeeAmount } from '../constants';
import { Tick, TickConstructorArgs } from './tick';
import { TickDataProvider } from './tickDataProvider';
import { ethers } from 'ethers';
interface SnapshotCumulativeInside {
    tickCumulativeInside: bigint;
    secondsPerLiquidityInsideX128: bigint;
    secondsInside: bigint;
}
interface ObserveResponse {
    tickCumulatives: bigint[];
    secondsPerLiquidityCumulativeX128s: bigint[];
}
export interface TransactionOverrides {
    gasPrice?: BigInt;
    gasLimit?: BigInt;
    value?: BigInt;
    nonce?: BigInt;
}
interface Slot0Response {
    sqrtPriceX96: BigInt;
    tick: number;
    observationIndex: number;
    observationCardinality: number;
    observationCardinalityNext: number;
    feeProtocol: number;
    unlocked: boolean;
}
/**
 * Represents a V3 pool
 */
export declare class Pool {
    readonly token0: Token;
    readonly token1: Token;
    readonly fee: FeeAmount;
    get sqrtRatioX96(): JSBI;
    readonly _sqrtRatioX96: bigint;
    get liquidity(): JSBI;
    readonly _liquidity: bigint;
    readonly tickCurrent: number;
    get tickDataProvider(): TickDataProvider;
    private _tickDataProvider;
    private _token0Price?;
    private _token1Price?;
    readonly _provider?: ethers.providers.Provider;
    static getAddress(tokenA: Token, tokenB: Token, fee: FeeAmount, initCodeHashManualOverride?: string, factoryAddressOverride?: string): string;
    /**
     * Initialize a pool from the latest chain data.
     *
     * @param provider The provider to fetch data from.
     * @param tokenA First token address of the pool
     * @param tokenB Second token address of the pool
     * @param fee The fee of the pool to fetch
     * @param poolAddress The pool address, optional, will be calculated if not given
     * @param initCodeHashManualOverride Init code hash override for pool address calculation. Used if pool address not given.
     * @param factoryAddressOverride Factory address override for pool address calculation. Used if pool address not given.
     * @returns The initialized Pool instance.
     */
    static initFromChain({ provider, tokenA, tokenB, fee, poolAddress, initCodeHashManualOverride, factoryAddressOverride }: {
        provider: ethers.providers.Provider;
        tokenA: Token;
        tokenB: Token;
        fee: FeeAmount;
        poolAddress?: string;
        initCodeHashManualOverride?: string;
        factoryAddressOverride?: string;
    }): Promise<Pool>;
    /**
     * Construct a pool
     * @param tokenA One of the tokens in the pool
     * @param tokenB The other token in the pool
     * @param fee The fee in hundredths of a bips of the input amount of every swap that is collected by the pool
     * @param sqrtRatioX96 The sqrt of the current ratio of amounts of token1 to token0
     * @param liquidity The current value of in range liquidity
     * @param tickCurrent The current tick of the pool
     * @param ticks The current state of the pool ticks or a data provider that can return tick data
     */
    constructor(tokenA: Token, tokenB: Token, fee: FeeAmount, sqrtRatioX96: BigintIsh, liquidity: BigintIsh, tickCurrent: number, _ticks?: TickDataProvider | (Tick | TickConstructorArgs)[], provider?: ethers.providers.Provider);
    /**
     * Returns true if the token is either token0 or token1
     * @param token The token to check
     * @returns True if token is either token0 or token
     */
    involvesToken(token: Token): boolean;
    /**
     * Returns the current mid price of the pool in terms of token0, i.e. the ratio of token1 over token0
     */
    get token0Price(): Price<Token, Token>;
    /**
     * Returns the current mid price of the pool in terms of token1, i.e. the ratio of token0 over token1
     */
    get token1Price(): Price<Token, Token>;
    /**
     * Return the price of the given token in terms of the other token in the pool.
     * @param token The token to return price of
     * @returns The price of the given token, in terms of the other.
     */
    priceOf(token: Token): Price<Token, Token>;
    /**
     * Returns the chain ID of the tokens in the pool.
     */
    get chainId(): number;
    /**
     * Given an input amount of a token, return the computed output amount, and a pool with state updated after the trade
     * @param inputAmount The input amount for which to quote the output amount
     * @param sqrtPriceLimitX96 The Q64.96 sqrt price limit
     * @returns The output amount and the pool with updated state
     */
    getOutputAmount(inputAmount: CurrencyAmount<Token>, sqrtPriceLimitX96?: bigint | JSBI): Promise<[CurrencyAmount<Token>, Pool]>;
    /**
     * Given a desired output amount of a token, return the computed input amount and a pool with state updated after the trade
     * @param outputAmount the output amount for which to quote the input amount
     * @param sqrtPriceLimitX96 The Q64.96 sqrt price limit. If zero for one, the price cannot be less than this value after the swap. If one for zero, the price cannot be greater than this value after the swap
     * @returns The input amount and the pool with updated state
     */
    getInputAmount(outputAmount: CurrencyAmount<Token>, sqrtPriceLimitX96?: bigint | JSBI): Promise<[CurrencyAmount<Token>, Pool]>;
    /**
     * Executes a swap
     * @param zeroForOne Whether the amount in is token0 or token1
     * @param amountSpecified The amount of the swap, which implicitly configures the swap as exact input (positive), or exact output (negative)
     * @param sqrtPriceLimitX96 The Q64.96 sqrt price limit. If zero for one, the price cannot be less than this value after the swap. If one for zero, the price cannot be greater than this value after the swap
     * @returns amountCalculated
     * @returns sqrtRatioX96
     * @returns liquidity
     * @returns tickCurrent
     */
    private swap;
    get tickSpacing(): number;
    initializeTicks(provider?: ethers.providers.Provider | undefined): Promise<void>;
    initializeTicksFromRpc(): Promise<void>;
    /**
     * Create a new Uniswap V3 pool with the given details (this will cost gas).
     *
     * @param _signer The wallet to use to sign the transaction.
     * @param provider The provider to use to propagate the transaction.
     * @param tokenA First token of the pool
     * @param tokenB Second token of the pool
     * @param fee The fee amount of the pool
     * @param transactionOverrides In case you want to override details of the transaction like gas, nonce, etc. optional.
     * @param factoryAddress The factory address to use. Only customize if you are using a fork. optional.
     * @returns The transaction response. You will still need to wait for tx inclusion.
     */
    static rpcCreatePool({ _signer, provider, tokenA, tokenB, fee, transactionOverrides, factoryAddress }: {
        _signer: ethers.Signer;
        provider: ethers.providers.Provider;
        tokenA: string;
        tokenB: string;
        fee: FeeAmount;
        transactionOverrides?: TransactionOverrides;
        factoryAddress?: string;
    }): Promise<ethers.providers.TransactionResponse>;
    static ethersTransactionOverrides(transactionOverrides?: TransactionOverrides): any;
    rpcContract(params: {
        poolAddress?: string;
        signer?: ethers.Signer;
    } | undefined): ethers.Contract;
    /**
     * Returns whether this pool exists on-chain.
     * @param poolAddress The address to check. optional.
     * @param blockNum The block number at which to check. Latest is assumed.
     * @returns true if the pool exists on-chain. false otherwise.
     */
    rpcPoolExists(options?: {
        poolAddress?: string;
        blockNum?: number;
    }): Promise<boolean>;
    /**
     * Returns the slot0 value of the pool from on-chain data.
     *
     * @param poolAddress Optional. The pool address.
     * @param blockNum Optional. The block number at which to fetch slot0. Latest is assumed.
     * @returns Slot0Response.
     */
    rpcSlot0(options?: {
        poolAddress?: string;
        blockNum?: number;
    }): Promise<Slot0Response>;
    /**
     * Returns the snapshot cumulative inside the pool from on-chain data.
     *
     * @param tickLower The lower tick to include into the range
     * @param tickUpper The upper tick to include into the range
     * @param poolAddress The pool address. optional.
     * @param blockNum The block number at which to fetch. optional. Latest is assumed.
     * @returns SnapshotCumulativeInside values.
     */
    rpcSnapshotCumulativesInside({ tickLower, tickUpper, poolAddress, blockNum }: {
        tickLower: number;
        tickUpper: number;
        poolAddress?: string;
        blockNum?: number;
    }): Promise<SnapshotCumulativeInside>;
    /**
     * Observation responses of the pool oracle.
     *
     * @param secondsAgo Array of timings for the oracle values.
     * @param poolAddress The pool address. optional.
     * @param blockNum The block number to fetch. optional.
     * @returns ObserveResponse values.
     */
    rpcObserve({ secondsAgo, poolAddress, blockNum }: {
        secondsAgo: number[];
        poolAddress?: string;
        blockNum?: number;
    }): Promise<ObserveResponse>;
    /**
     * Increase the observation cardinality for this pool (this tx requires gas).
     *
     * @param signer The signer to use to sign the transaction.
     * @param observationCardinalityNext The next cardinality (you pay for).
     * @param poolAddress The pool address. optional.
     * @param transactionOverrides If you want to customize gas, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for tx inclusion.
     */
    rpcIncreaseObservationCardinalityNext({ signer, observationCardinalityNext, poolAddress, transactionOverrides }: {
        signer: ethers.Signer;
        observationCardinalityNext: number;
        poolAddress?: string;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
    /**
     * Collects tokens owed to a position
     *
     * Does not recompute fees earned, which must be done either via mint or burn of any amount of liquidity.
     * Collect must be called by the position owner. To withdraw only token0 or only token1, amount0Requested or amount1Requested may be set to zero.
     * To withdraw all tokens owed, caller may pass any value greater than the actual tokens owed, e.g. type(uint128).max.
     * Tokens owed may be from accumulated swap fees or burned liquidity.
     *
     * This tx requires gas.
     *
     * @param signer The signer to use to sign the transaction.
     * @param recipient The recipient of the fees.
     * @param tickLower The lower tick of the position for which to collect fees
     * @param tickUpper The upper tick of the position for which to collect fees
     * @param amount0Requested How much token0 should be withdrawn from the fees owed
     * @param amount1Requested How much token1 should be withdrawn from the fees owed
     * @param poolAddress The pool address. optional.
     * @param transactionOverrides If you want to customize gas, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for tx inclusion.
     */
    rpcCollect({ signer, recipient, tickLower, tickUpper, amount0Requested, amount1Requested, poolAddress, transactionOverrides }: {
        signer: ethers.Signer;
        recipient: string;
        tickLower: number;
        tickUpper: number;
        amount0Requested: BigInt;
        amount1Requested: BigInt;
        poolAddress?: string;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
    /**
     * Burn liquidity from the sender and account tokens owed for the liquidity to the position.
     * This tx requires gas.
     *
     * @param signer The signer to use to sign the tx.
     * @param tickLower The lower tick of the position for which to burn liquidity
     * @param tickUpper The upper tick of the position for which to burn liquidity
     * @param amount How much liquidity to burn
     * @param poolAddress The pool address. optional.
     * @param transactionOverrides If you want to customize gas, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for tx inclusion.
     */
    rpcBurn({ signer, tickLower, tickUpper, amount, poolAddress, transactionOverrides }: {
        signer: ethers.Signer;
        tickLower: number;
        tickUpper: number;
        amount: BigInt;
        poolAddress?: string;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
}
export {};
