import { BigintIsh, Percent, CurrencyAmount, Currency, NativeCurrency } from '@uniswap/sdk-core';
import { Position } from './entities/position';
import { MethodParameters } from './utils/calldata';
import { Interface } from '@ethersproject/abi';
import { PermitOptions } from './selfPermit';
import { Pool, TransactionOverrides } from './entities';
import { ethers } from 'ethers';
export interface MintSpecificOptions {
    /**
     * The account that should receive the minted NFT.
     */
    recipient: string;
    /**
     * Creates pool if not initialized before mint.
     */
    createPool?: boolean;
}
export interface IncreaseSpecificOptions {
    /**
     * Indicates the ID of the position to increase liquidity for.
     */
    tokenId: BigintIsh;
}
/**
 * Options for producing the calldata to add liquidity.
 */
export interface CommonAddLiquidityOptions {
    /**
     * How much the pool price is allowed to move.
     */
    slippageTolerance: Percent;
    /**
     * When the transaction expires, in epoch seconds.
     */
    deadline: BigintIsh;
    /**
     * Whether to spend ether. If true, one of the pool tokens must be WETH, by default false
     */
    useNative?: NativeCurrency;
    /**
     * The optional permit parameters for spending token0
     */
    token0Permit?: PermitOptions;
    /**
     * The optional permit parameters for spending token1
     */
    token1Permit?: PermitOptions;
}
export type MintOptions = CommonAddLiquidityOptions & MintSpecificOptions;
export type IncreaseOptions = CommonAddLiquidityOptions & IncreaseSpecificOptions;
export type AddLiquidityOptions = MintOptions | IncreaseOptions;
export interface SafeTransferOptions {
    /**
     * The account sending the NFT.
     */
    sender: string;
    /**
     * The account that should receive the NFT.
     */
    recipient: string;
    /**
     * The id of the token being sent.
     */
    tokenId: BigintIsh;
    /**
     * The optional parameter that passes data to the `onERC721Received` call for the staker
     */
    data?: string;
}
export interface CollectOptions {
    /**
     * Indicates the ID of the position to collect for.
     */
    tokenId: BigintIsh;
    /**
     * Expected value of tokensOwed0, including as-of-yet-unaccounted-for fees/liquidity value to be burned
     */
    expectedCurrencyOwed0: CurrencyAmount<Currency>;
    /**
     * Expected value of tokensOwed1, including as-of-yet-unaccounted-for fees/liquidity value to be burned
     */
    expectedCurrencyOwed1: CurrencyAmount<Currency>;
    /**
     * The account that should receive the tokens.
     */
    recipient: string;
}
export interface NFTPermitOptions {
    v: 0 | 1 | 27 | 28;
    r: string;
    s: string;
    deadline: BigintIsh;
    spender: string;
}
/**
 * Options for producing the calldata to exit a position.
 */
export interface RemoveLiquidityOptions {
    /**
     * The ID of the token to exit
     */
    tokenId: BigintIsh;
    /**
     * The percentage of position liquidity to exit.
     */
    liquidityPercentage: Percent;
    /**
     * How much the pool price is allowed to move.
     */
    slippageTolerance: Percent;
    /**
     * When the transaction expires, in epoch seconds.
     */
    deadline: BigintIsh;
    /**
     * Whether the NFT should be burned if the entire position is being exited, by default false.
     */
    burnToken?: boolean;
    /**
     * The optional permit of the token ID being exited, in case the exit transaction is being sent by an account that does not own the NFT
     */
    permit?: NFTPermitOptions;
    /**
     * Parameters to be passed on to collect
     */
    collectOptions: Omit<CollectOptions, 'tokenId'>;
}
export declare abstract class NonfungiblePositionManager {
    static INTERFACE: Interface;
    /**
     * Cannot be constructed.
     */
    private constructor();
    private static encodeCreate;
    /**
     * Create and if necessary initialize the pool.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param pool The pool to create and initialize.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static createOnChain(_signer: ethers.Signer, provider: ethers.providers.Provider, pool: Pool, transactionOverrides?: TransactionOverrides): Promise<ethers.providers.TransactionResponse>;
    static createCallParameters(pool: Pool): MethodParameters;
    /**
     * Creates a new position and mints the NFT of the position on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param position The position to add liquidity to.
     * @param options Mint liquidity Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static createPositionOnChain(_signer: ethers.Signer, provider: ethers.providers.Provider, position: Position, options: MintOptions, transactionOverrides?: TransactionOverrides): Promise<ethers.providers.TransactionResponse>;
    /**
     * Add liquidity to position on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param position The position to add liquidity to.
     * @param options Increase liquidity Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static increasePositionOnChain(_signer: ethers.Signer, provider: ethers.providers.Provider, position: Position, options: IncreaseOptions, transactionOverrides?: TransactionOverrides): Promise<ethers.providers.TransactionResponse>;
    /**
     * Returns calldata for creating a position and minting the NFT.
     *
     * @param position The position to create.
     * @param options The mint options
     * @returns calldata and value to be used for signing and sending a transaction.
     */
    static createPositionCallParameters(position: Position, options: MintOptions): MethodParameters;
    /**
     * Returns calldata for increasing an existing position.
     *
     * @param position The position to create.
     * @param options The increase options
     * @returns calldata and value to be used for signing and sending a transaction.
     */
    static increasePositionCallParameters(position: Position, options: IncreaseOptions): MethodParameters;
    static addCallParameters(position: Position, options: AddLiquidityOptions): MethodParameters;
    private static encodeCollect;
    /**
     * Collect fees on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param options Collect Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static collectOnChain(_signer: ethers.Signer, provider: ethers.providers.Provider, options: CollectOptions, transactionOverrides?: TransactionOverrides): Promise<ethers.providers.TransactionResponse>;
    static collectCallParameters(options: CollectOptions): MethodParameters;
    /**
     * Remove a position on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param position The position to remove.
     * @param options Remove Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static removeOnChain(_signer: ethers.Signer, provider: ethers.providers.Provider, position: Position, options: RemoveLiquidityOptions, transactionOverrides?: TransactionOverrides): Promise<ethers.providers.TransactionResponse>;
    /**
     * Produces the calldata for completely or partially exiting a position
     * @param position The position to exit
     * @param options Additional information necessary for generating the calldata
     * @returns The call parameters
     */
    static removeCallParameters(position: Position, options: RemoveLiquidityOptions): MethodParameters;
    /**
     * Safe transfer from on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param options SafeTransferOptions.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static safeTransferFromOnChain(_signer: ethers.Signer, provider: ethers.providers.Provider, options: SafeTransferOptions, transactionOverrides?: TransactionOverrides): Promise<ethers.providers.TransactionResponse>;
    static safeTransferFromParameters(options: SafeTransferOptions): MethodParameters;
    /**
     * Returns a URI describing a particular positionId.
     *
     * @param provider The provider to fetch from.
     * @param positionId The tokenId to fetch.
     *
     * @returns The positionId URI as a base64 string with URI data.
     */
    static tokenURI(provider: ethers.providers.Provider, positionId: BigintIsh): Promise<string>;
    /**
     * Returns the base URI of positions.
     *
     * @param provider The provider to fetch from.
     *
     * @returns The base URI.
     */
    static baseURI(provider: ethers.providers.Provider): Promise<string>;
}
